@@ -60,16 +60,14 @@ HRESULT WINAPI D3D12GetDebugInterface(REFIID iid, void **debug)
 HRESULT WINAPI D3D12CreateDeviceVKD3D(IUnknown *adapter, D3D_FEATURE_LEVEL minimum_feature_level,
         REFIID iid, void **device, enum vkd3d_api_version api_version)
 {
-    struct vkd3d_optional_instance_extensions_info optional_extensions_info;
+    struct vkd3d_optional_instance_extensions_info optional_instance_extensions_info;
+    struct vkd3d_optional_device_extensions_info optional_device_extensions_info;
     struct vkd3d_instance_create_info instance_create_info;
     struct vkd3d_device_create_info device_create_info;
 
-    static const char * const instance_extensions[] =
-    {
-        VK_KHR_SURFACE_EXTENSION_NAME,
-    };
     static const char * const optional_instance_extensions[] =
     {
+        VK_KHR_SURFACE_EXTENSION_NAME,
         "VK_KHR_android_surface",
         "VK_KHR_wayland_surface",
         "VK_KHR_win32_surface",
@@ -79,7 +77,7 @@ HRESULT WINAPI D3D12CreateDeviceVKD3D(IUnknown *adapter, D3D_FEATURE_LEVEL minim
         "VK_MVK_macos_surface",
         "VK_MVK_ios_surface",
     };
-    static const char * const device_extensions[] =
+    static const char * const optional_device_extensions[] =
     {
         VK_KHR_SWAPCHAIN_EXTENSION_NAME,
     };
@@ -95,27 +93,28 @@ HRESULT WINAPI D3D12CreateDeviceVKD3D(IUnknown *adapter, D3D_FEATURE_LEVEL minim
     if (adapter)
         FIXME("Ignoring adapter %p.\n", adapter);
 
-    memset(&optional_extensions_info, 0, sizeof(optional_extensions_info));
-    optional_extensions_info.type = VKD3D_STRUCTURE_TYPE_OPTIONAL_INSTANCE_EXTENSIONS_INFO;
-    optional_extensions_info.next = &application_info;
-    optional_extensions_info.extensions = optional_instance_extensions;
-    optional_extensions_info.extension_count = ARRAY_SIZE(optional_instance_extensions);
+    memset(&optional_instance_extensions_info, 0, sizeof(optional_instance_extensions_info));
+    optional_instance_extensions_info.type = VKD3D_STRUCTURE_TYPE_OPTIONAL_INSTANCE_EXTENSIONS_INFO;
+    optional_instance_extensions_info.next = &application_info;
+    optional_instance_extensions_info.extensions = optional_instance_extensions;
+    optional_instance_extensions_info.extension_count = ARRAY_SIZE(optional_instance_extensions);
 
     memset(&instance_create_info, 0, sizeof(instance_create_info));
     instance_create_info.type = VKD3D_STRUCTURE_TYPE_INSTANCE_CREATE_INFO;
-    instance_create_info.next = &optional_extensions_info;
+    instance_create_info.next = &optional_instance_extensions_info;
     instance_create_info.pfn_signal_event = vkd3d_signal_event;
     instance_create_info.wchar_size = sizeof(WCHAR);
-    instance_create_info.instance_extensions = instance_extensions;
-    instance_create_info.instance_extension_count = ARRAY_SIZE(instance_extensions);
+
+    memset(&optional_device_extensions_info, 0, sizeof(optional_device_extensions_info));
+    optional_device_extensions_info.type = VKD3D_STRUCTURE_TYPE_OPTIONAL_DEVICE_EXTENSIONS_INFO;
+    optional_device_extensions_info.extensions = optional_device_extensions;
+    optional_device_extensions_info.extension_count = ARRAY_SIZE(optional_device_extensions);
 
     memset(&device_create_info, 0, sizeof(device_create_info));
     device_create_info.type = VKD3D_STRUCTURE_TYPE_DEVICE_CREATE_INFO;
-    device_create_info.next = NULL;
+    device_create_info.next = &optional_device_extensions_info;
     device_create_info.minimum_feature_level = minimum_feature_level;
     device_create_info.instance_create_info = &instance_create_info;
-    device_create_info.device_extensions = device_extensions;
-    device_create_info.device_extension_count = ARRAY_SIZE(device_extensions);
 
     return vkd3d_create_device(&device_create_info, iid, device);
 }
@@ -236,15 +235,15 @@ static enum vkd3d_shader_target_type get_target_for_profile(const char *profile)
     return VKD3D_SHADER_TARGET_DXBC_TPF;
 }
 
-HRESULT WINAPI D3DCompile2(const void *data, SIZE_T data_size, const char *filename,
+HRESULT WINAPI D3DCompile2VKD3D(const void *data, SIZE_T data_size, const char *filename,
         const D3D_SHADER_MACRO *macros, ID3DInclude *include, const char *entry_point,
         const char *profile, UINT flags, UINT effect_flags, UINT secondary_flags,
         const void *secondary_data, SIZE_T secondary_data_size, ID3DBlob **shader_blob,
-        ID3DBlob **messages_blob)
+        ID3DBlob **messages_blob, unsigned int compiler_version)
 {
     struct vkd3d_shader_preprocess_info preprocess_info;
     struct vkd3d_shader_hlsl_source_info hlsl_info;
-    struct vkd3d_shader_compile_option options[4];
+    struct vkd3d_shader_compile_option options[7];
     struct vkd3d_shader_compile_info compile_info;
     struct vkd3d_shader_compile_option *option;
     struct vkd3d_shader_code byte_code;
@@ -255,14 +254,14 @@ HRESULT WINAPI D3DCompile2(const void *data, SIZE_T data_size, const char *filen
 
     TRACE("data %p, data_size %"PRIuPTR", filename %s, macros %p, include %p, entry_point %s, "
             "profile %s, flags %#x, effect_flags %#x, secondary_flags %#x, secondary_data %p, "
-            "secondary_data_size %"PRIuPTR", shader_blob %p, messages_blob %p.\n",
+            "secondary_data_size %"PRIuPTR", shader_blob %p, messages_blob %p, compiler_version %u.\n",
             data, (uintptr_t)data_size, debugstr_a(filename), macros, include, debugstr_a(entry_point),
             debugstr_a(profile), flags, effect_flags, secondary_flags, secondary_data,
-            (uintptr_t)secondary_data_size, shader_blob, messages_blob);
+            (uintptr_t)secondary_data_size, shader_blob, messages_blob, compiler_version);
 
     if (flags & ~(D3DCOMPILE_DEBUG | D3DCOMPILE_PACK_MATRIX_ROW_MAJOR | D3DCOMPILE_PACK_MATRIX_COLUMN_MAJOR))
         FIXME("Ignoring flags %#x.\n", flags);
-    if (effect_flags)
+    if (effect_flags & ~D3DCOMPILE_EFFECT_CHILD_EFFECT)
         FIXME("Ignoring effect flags %#x.\n", effect_flags);
     if (secondary_flags)
         FIXME("Ignoring secondary flags %#x.\n", secondary_flags);
@@ -272,7 +271,7 @@ HRESULT WINAPI D3DCompile2(const void *data, SIZE_T data_size, const char *filen
 
     option = &options[0];
     option->name = VKD3D_SHADER_COMPILE_OPTION_API_VERSION;
-    option->value = VKD3D_SHADER_API_VERSION_1_11;
+    option->value = VKD3D_SHADER_API_VERSION_1_17;
 
     compile_info.type = VKD3D_SHADER_STRUCTURE_TYPE_COMPILE_INFO;
     compile_info.next = &preprocess_info;
@@ -319,7 +318,7 @@ HRESULT WINAPI D3DCompile2(const void *data, SIZE_T data_size, const char *filen
         option->value = 0;
         if (flags & D3DCOMPILE_PACK_MATRIX_ROW_MAJOR)
             option->value |= VKD3D_SHADER_COMPILE_OPTION_PACK_MATRIX_ROW_MAJOR;
-        if (flags & D3DCOMPILE_PACK_MATRIX_COLUMN_MAJOR)
+        else if (flags & D3DCOMPILE_PACK_MATRIX_COLUMN_MAJOR)
             option->value |= VKD3D_SHADER_COMPILE_OPTION_PACK_MATRIX_COLUMN_MAJOR;
     }
 
@@ -330,6 +329,27 @@ HRESULT WINAPI D3DCompile2(const void *data, SIZE_T data_size, const char *filen
         option->value = VKD3D_SHADER_COMPILE_OPTION_BACKCOMPAT_MAP_SEMANTIC_NAMES;
     }
 
+    if (effect_flags & D3DCOMPILE_EFFECT_CHILD_EFFECT)
+    {
+        option = &options[compile_info.option_count++];
+        option->name = VKD3D_SHADER_COMPILE_OPTION_CHILD_EFFECT;
+        option->value = true;
+    }
+
+    if (compiler_version <= 39)
+    {
+        option = &options[compile_info.option_count++];
+        option->name = VKD3D_SHADER_COMPILE_OPTION_INCLUDE_EMPTY_BUFFERS_IN_EFFECTS;
+        option->value = true;
+    }
+
+    if (compiler_version < 42)
+    {
+        option = &options[compile_info.option_count++];
+        option->name = VKD3D_SHADER_COMPILE_OPTION_WARN_IMPLICIT_TRUNCATION;
+        option->value = false;
+    }
+
     ret = vkd3d_shader_compile(&compile_info, &byte_code, &messages);
 
     if (messages && messages_blob)
@@ -370,6 +390,24 @@ HRESULT WINAPI D3DCompile2(const void *data, SIZE_T data_size, const char *filen
     return hresult_from_vkd3d_result(ret);
 }
 
+HRESULT WINAPI D3DCompile2(const void *data, SIZE_T data_size, const char *filename,
+        const D3D_SHADER_MACRO *macros, ID3DInclude *include, const char *entry_point,
+        const char *profile, UINT flags, UINT effect_flags, UINT secondary_flags,
+        const void *secondary_data, SIZE_T secondary_data_size, ID3DBlob **shader_blob,
+        ID3DBlob **messages_blob)
+{
+    TRACE("data %p, data_size %"PRIuPTR", filename %s, macros %p, include %p, entry_point %s, "
+            "profile %s, flags %#x, effect_flags %#x, secondary_flags %#x, secondary_data %p, "
+            "secondary_data_size %"PRIuPTR", shader_blob %p, messages_blob %p.\n",
+            data, (uintptr_t)data_size, debugstr_a(filename), macros, include, debugstr_a(entry_point),
+            debugstr_a(profile), flags, effect_flags, secondary_flags, secondary_data,
+            (uintptr_t)secondary_data_size, shader_blob, messages_blob);
+
+    return D3DCompile2VKD3D(data, data_size, filename, macros, include,
+            entry_point, profile, flags, effect_flags, secondary_flags,
+            secondary_data, secondary_data_size, shader_blob, messages_blob, 47);
+}
+
 HRESULT WINAPI D3DCompile(const void *data, SIZE_T data_size, const char *filename,
         const D3D_SHADER_MACRO *macros, ID3DInclude *include, const char *entrypoint,
         const char *profile, UINT flags, UINT effect_flags, ID3DBlob **shader, ID3DBlob **error_messages)
@@ -397,7 +435,7 @@ HRESULT WINAPI D3DPreprocess(const void *data, SIZE_T size, const char *filename
 
     static const struct vkd3d_shader_compile_option options[] =
     {
-        {VKD3D_SHADER_COMPILE_OPTION_API_VERSION, VKD3D_SHADER_API_VERSION_1_11},
+        {VKD3D_SHADER_COMPILE_OPTION_API_VERSION, VKD3D_SHADER_API_VERSION_1_17},
     };
 
     TRACE("data %p, size %"PRIuPTR", filename %s, macros %p, include %p, preprocessed_blob %p, messages_blob %p.\n",
@@ -773,7 +811,7 @@ static HRESULT get_blob_part(const void *data, SIZE_T data_size,
     {
         dst_dxbc = sections[0].data;
     }
-    else if ((ret = vkd3d_shader_serialize_dxbc(section_count, sections, &dst_dxbc, NULL) < 0))
+    else if ((ret = vkd3d_shader_serialize_dxbc(section_count, sections, &dst_dxbc, NULL)) < 0)
     {
         WARN("Failed to serialise DXBC, ret %d.\n", ret);
         hr = hresult_from_vkd3d_result(ret);
@@ -904,7 +942,7 @@ HRESULT WINAPI D3DStripShader(const void *data, SIZE_T data_size, UINT flags, ID
             sections[section_count++] = *src_section;
     }
 
-    if ((ret = vkd3d_shader_serialize_dxbc(section_count, sections, &dst_dxbc, NULL) < 0))
+    if ((ret = vkd3d_shader_serialize_dxbc(section_count, sections, &dst_dxbc, NULL)) < 0)
     {
         WARN("Failed to serialise DXBC, ret %d.\n", ret);
         hr = hresult_from_vkd3d_result(ret);
@@ -943,7 +981,7 @@ HRESULT WINAPI D3DDisassemble(const void *data, SIZE_T data_size,
 
     static const struct vkd3d_shader_compile_option options[] =
     {
-        {VKD3D_SHADER_COMPILE_OPTION_API_VERSION, VKD3D_SHADER_API_VERSION_1_11},
+        {VKD3D_SHADER_COMPILE_OPTION_API_VERSION, VKD3D_SHADER_API_VERSION_1_17},
     };
 
     TRACE("data %p, data_size %"PRIuPTR", flags %#x, comments %p, blob %p.\n",
